#include "prefix.h"
#include "zero.h"

void *KDevice::operator new(size_t, PDRIVER_OBJECT DriverObject, PCWSTR DeviceName, PCWSTR SymLinkName)
{ 
 // Create device object
 KDevice *Device=0;
 NTSTATUS ConstructorStatus;
 UNICODE_STRING Name;
 RtlInitUnicodeString(&Name, DeviceName);

 ConstructorStatus = IoCreateDevice(DriverObject,
                                  sizeof(KDevice)-sizeof(DEVICE_OBJECT),
                                  &Name,
                                  FILE_DEVICE_UNKNOWN,
                                  FILE_READ_ONLY_DEVICE/*|FILE_DEVICE_SECURE_OPEN*/,
                                  FALSE,
                                  (PDEVICE_OBJECT*)&Device);
 if(NT_SUCCESS(ConstructorStatus))
  Device->ConstructorStatus=ConstructorStatus;
 else
  return 0;

 if(SymLinkName)
 {
  RtlInitUnicodeString(&Device->SymLink, SymLinkName);
  Device->ConstructorStatus=IoCreateSymbolicLink(&Device->SymLink, &Name);
 }
 return Device;
}

KDevice::~KDevice()
{
 if(SymLink.Length)
  IoDeleteSymbolicLink(&SymLink);
 IoDeleteDevice(this);
}

NTSTATUS KDevice::Dispatch(PDEVICE_OBJECT DeviceObject, PIRP Irp)
{
 PIO_STACK_LOCATION IrpSp;

// KdBreakPoint();

 printk("KDevice::Dispatch()->");
 KDevice *Device=(KDevice *)DeviceObject;

 IrpSp=IoGetCurrentIrpStackLocation(Irp);

 switch(IrpSp->MajorFunction) 
 {
  case IRP_MJ_CREATE:
   return Device->Create(Irp);
  case IRP_MJ_CLOSE:
   return Device->Close(Irp);
  case IRP_MJ_READ:
   return Device->Read(Irp);
  default:
   printk("InvalidRequest 0x%08x\n", IrpSp->MajorFunction);
   Irp->IoStatus.Status = STATUS_INVALID_DEVICE_REQUEST;
   IoCompleteRequest(Irp, IO_NO_INCREMENT);
   return STATUS_INVALID_DEVICE_REQUEST;
 }
}
